package animation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.LayerManager;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IPage;

import roneditor.AbstractMultiViewerPage;
import roneditor.AbstractPageBookView;
import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;

/**
 * @author Tony Modica
 * 
 */
public class AnimatingCommand extends Command {
	// TODO AnimatingCommand als CompoundCommand erweitern, sodass alles
	// automatisch passiert!
	// Fr jedes Command, das hinzugefgt wird, eine eigene Animation erzeugen,
	// die vor dem Command gezeigt wird!
	// Zustzlich aber eine abschlieende Animation ermglichen!
	// Am besten fr jedes Command die Stepzahl merken und ausfhren, wenn der
	// Step durchgefhrt wurde!

	private final HashMap<EObject, AnimatedElement> animatedElementsMap;

	private final List<AnimatedElement> specifiedElements;

	private final List<Double> relativeDurations;

	/**
	 * Default duration for a single step is 400 msec.
	 */
	private static final int DEFAULT_DURATION = 400;

	public AnimatingCommand() {
		animatedElementsMap = new HashMap<EObject, AnimatedElement>();
		specifiedElements = new ArrayList<AnimatedElement>();
		relativeDurations = new ArrayList<Double>();
		// the first step has to be performed immediately
		relativeDurations.add(0.0);
	}

	public AnimatingCommand(final String ID) {
		super(ID);
		animatedElementsMap = new HashMap<EObject, AnimatedElement>();
		specifiedElements = new ArrayList<AnimatedElement>();
		relativeDurations = new ArrayList<Double>();
		// the first step has to be performed immediately
		relativeDurations.add(0.0);
	}

	@Override
	public void execute() {
		performAnimation(-1, false);
	}

	@Override
	public void undo() {
		performAnimation(-1, true);
	}

	public void clean() {
		animatedElementsMap.clear();
		specifiedElements.clear();
		relativeDurations.clear();
		// the first step has to be performed immediately
		relativeDurations.add(0.0);
	}

	protected void performAnimation(int completeDuration, final boolean isUndo) {
		// complete step definition if needed
		if (!specifiedElements.isEmpty()) {
			// fill up initialized animated elements that have not been extended
			// by
			// a step specification
			final List<AnimatedElement> unspecified = new ArrayList<AnimatedElement>(
					animatedElementsMap.values());
			unspecified.removeAll(specifiedElements);
			for (final AnimatedElement animatedElement : unspecified) {
				animatedElement.addPlaceholderStep();
			}
		}

		for (final AnimatedElement element : animatedElementsMap.values()) {
			element.prepareForAnimation();
		}

		// calculate sum of all relative step durations
		double durationSum = 0.0;
		for (final Double relativeDuration : relativeDurations) {
			durationSum += relativeDuration;
		}

		// if complete duration is not specified by user assume default duration
		// for each relative duration
		if (completeDuration == -1) {
			// count 1 relative duration as 1 default duration
			completeDuration = (int) (DEFAULT_DURATION * durationSum);
		}

		// calculate absolute durations in msec
		final List<Integer> absoluteDurations = new ArrayList<Integer>();
		for (int i = 0; i < relativeDurations.size(); i++) {
			final double absoluteDuration = relativeDurations.get(i)
					* completeDuration / durationSum;
			absoluteDurations.add((int) absoluteDuration);
		}

		if (isUndo) {
			performStep(getStepCounter() - 1, 0);
			for (int i = getStepCounter() - 2; i >= 0; i--) {
				performStep(i, absoluteDurations.get(i + 1));
			}
		} else {
			for (int i = 0; i < getStepCounter(); i++) {
				performStep(i, absoluteDurations.get(i));
			}
		}

		for (final AnimatedElement element : animatedElementsMap.values()) {
			element.animationDone();
		}
	}

	private void performStep(final int i, final int duration) {
		MultipleAnimation.markBegin();
		for (final AnimatedElement element : animatedElementsMap.values()) {
			element.prepareStep(i);
		}
		MultipleAnimation.run(duration);
	}

	public void nextStep() {
		nextStep(1);
	}

	public void nextStep(final double relativeDuration) {
		// fill up initialized animated elements that have not been extended by
		// a step specification
		final List<AnimatedElement> unspecified = new ArrayList<AnimatedElement>(
				animatedElementsMap.values());
		unspecified.removeAll(specifiedElements);
		for (final AnimatedElement animatedElement : unspecified) {
			animatedElement.addPlaceholderStep();
		}

		// save specified relative duration for next step
		if (relativeDuration <= 0) {
			throw new IllegalArgumentException(
					"A relative duration for a step has to be greater than 0!");
		}
		relativeDurations.add(relativeDuration);

		// prepare for next step
		specifiedElements.clear();
	}

	public int getStepCounter() {
		return relativeDurations.size();
	}

	public void initializeAnimatedElement(final Object modelOrEditPart,
			final Object viewerOrTop) {
		Assert
				.isTrue(getStepCounter() == 1,
						"In this version all initialization have to be made in the first step!");
		final AnimatedElement newElement = new AnimatedElement(modelOrEditPart,
				viewerOrTop);
		animatedElementsMap.put(newElement.model, newElement);
	}

	public void specifyStep(final Object modelOrEditPart,
			final Object locationObject, final Dimension size) {
		getAndPrepareAnimatedElement(modelOrEditPart).addStep(locationObject,
				size);
	}

	public void specifyStep(final Object modelOrEditPart,
			final Object locationObject, final double sizeFactor) {
		getAndPrepareAnimatedElement(modelOrEditPart).addStep(locationObject,
				sizeFactor);
	}

	public void specifyStep(final Object modelOrEditPart,
			final Object locationObject) {
		getAndPrepareAnimatedElement(modelOrEditPart).addStep(locationObject);
	}

	private AnimatedElement getAndPrepareAnimatedElement(
			final Object modelOrEditPart) {
		AnimatedElement animatedElement = null;
		if (modelOrEditPart instanceof EObject) {
			animatedElement = animatedElementsMap.get(modelOrEditPart);
		} else if (modelOrEditPart instanceof GraphicalEditPart) {
			animatedElement = animatedElementsMap
					.get(((GraphicalEditPart) modelOrEditPart).getModel());
		} else {
			throw new IllegalArgumentException(
					"A step has to be specified for a EObject or an GraphicalEditPart!");
		}
		Assert.isNotNull(animatedElement,
				"AnimatedElement has not been initialized for "
						+ modelOrEditPart);
		Assert.isTrue(!specifiedElements.contains(animatedElement),
				"This step has already been defined for model "
						+ modelOrEditPart);
		specifiedElements.add(animatedElement);
		return animatedElement;
	}

	/**
	 * @author Tony Modica
	 * 
	 */
	public static class AnimatedElement {

		private final AnimatorForMultiple animator;

		/**
		 * The model element whose figure in a specific viewer is to be
		 * animated.
		 */
		private EObject model;

		/**
		 * A container model being an (indirect) parent of {@link #model}. The
		 * viewer having this container as contents will be used for animation.
		 */
		private EObject topModel;

		/**
		 * The zoom level of the viewer determined by the constructor. Used to
		 * scale absolute positions appropriately for viewers with different
		 * zoom levels.
		 */
		private double initialZoomLevel;

		/**
		 * A ArrayList of {@link Localizer}s defining the path of this animated
		 * element.
		 */
		private final ArrayList<Localizer> path;

		// transient data to perform the animation
		public EditPartViewer viewer;
		public IFigure figure;
		public IFigure originalParent;
		public Rectangle originalBounds;
		public IFigure layer;
		public double currentZoomLevel;
		public boolean canPerform = false;
		private Rectangle cachedFirstBounds;

		/**
		 * @param modelOrEditPart
		 * @param viewerOrTop
		 */
		public AnimatedElement(final Object modelOrEditPart,
				final Object viewerOrTop) {
			if (modelOrEditPart instanceof GraphicalEditPart) {
				model = (EObject) ((GraphicalEditPart) modelOrEditPart)
						.getModel();
			} else if (modelOrEditPart instanceof EObject) {
				model = (EObject) modelOrEditPart;
			} else {
				throw new IllegalArgumentException(
						"AnimatedElement must be initialized with an EObject or GraphicalEditPart!");
			}

			if (viewerOrTop instanceof EObject) {
				topModel = (EObject) viewerOrTop;
				final ScrollingGraphicalViewer viewer = ((ScrollingGraphicalViewer) findViewerShowing(topModel));
				final ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart) viewer
						.getRootEditPart();
				initialZoomLevel = root.getZoomManager().getZoom();
			} else if (viewerOrTop instanceof EditPartViewer) {
				final ScrollingGraphicalViewer viewer = (ScrollingGraphicalViewer) viewerOrTop;
				topModel = (EObject) viewer.getContents().getModel();
				final ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart) viewer
						.getRootEditPart();
				initialZoomLevel = root.getZoomManager().getZoom();
			} else {
				throw new IllegalArgumentException(
						"AnimatedElement must be initialized with an top level EObject or EditPartViewer!");
			}
			path = new ArrayList<Localizer>();
			animator = new AnimatorForMultiple();
		}

		/**
		 * Adds a {@link Localizer} relying on the passed object to the path of
		 * this animated element. This object is used to resolve its location in
		 * a viewer context. The size factor is fixed to 1 avoiding size change.
		 * 
		 * @param locationObject
		 *            If <code>null</code>, the added {@link Localizer} will
		 *            be interpolated completely.
		 * 
		 * @see Localizer#Localizer(Object)
		 */
		public void addStep(final Object locationObject) {
			if (locationObject == null) {
				// add placeholder to be interpolated
				path.add(new Localizer());
			} else {
				path.add(new Localizer(locationObject));
			}
		}

		/**
		 * Adds a {@link Localizer} relying on the passed objects to the path of
		 * this animated element. These objects are used to resolve a location
		 * and a size in a viewer context. See the {@link Localizer}
		 * constructors for details.
		 * 
		 * @param locationObject
		 * @param sizeObject
		 * 
		 * @see Localizer#Localizer(Object, Dimension)
		 * @see Localizer#Localizer(Object, double)
		 */
		public void addStep(final Object locationObject, final Object sizeObject) {
			if (sizeObject instanceof Double) {
				path.add(new Localizer(locationObject, (Double) sizeObject));
			} else if (sizeObject instanceof Dimension) {
				path.add(new Localizer(locationObject, (Dimension) sizeObject));
			} else if (sizeObject == null) {
				path.add(new Localizer(locationObject, null));
			} else {
				throw new IllegalArgumentException(
						"Size of a step must be initialized with a Double or a Dimension!");
			}
		}

		/**
		 * Adds a completely interpolated {@link Localizer} to the path.
		 */
		public void addPlaceholderStep() {
			path.add(new Localizer());
		}

		/**
		 * 
		 */
		public void prepareForAnimation() {
			viewer = findViewerShowing(topModel);
			if (viewer == null) {
				canPerform = false;
				return;
			}

			final ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart) viewer
					.getRootEditPart();
			currentZoomLevel = root.getZoomManager().getZoom();

			figure = ((GraphicalEditPart) viewer.getEditPartRegistry().get(
					model)).getFigure();

			prepareLocalizers();

			for (final Localizer localizer : path) {
				if (localizer.getActualBounds() == null) {
					// This assert is not needed but may be useful to locate
					// errors
					// Assert.isNotNull(localizer, "Found null-localizer in list
					// of model " + model);
					canPerform = false;
					return;
				}
			}

			originalParent = figure.getParent();
			originalBounds = figure.getBounds().getCopy();

			// find a layer of the viewer by its root to hold the animated
			// figure
			layer = LayerManager.Helper.find(root).getLayer(
					LayerConstants.SCALABLE_LAYERS);

			layer.addLayoutListener(animator);
			layer.add(figure);
			canPerform = true;
		}

		/**
		 * 
		 */
		private void prepareLocalizers() {
			final int firstLocPos = getIndexOfNextNotInterpolated(0);
			final int lastLocPos = path.size() - 1;
			// TODO Fr AnimatingCompoundCommand: Es mssen undefinierte
			// Localizer vor dem StartLocalizer zugelassen werden
			// Ansonsten sollte firstLocPos erstmal immer 0 sein:
			Assert
					.isTrue(
							firstLocPos == 0,
							"First Localizer of "
									+ model
									+ " is interpolated and can not be used as concrete start Localizer!");
			// cache FirstLoc only once!
			if (null == cachedFirstBounds) {
				cachedFirstBounds = path.get(firstLocPos).resolveBounds(viewer,
						figure.getBounds().getSize());
			}
			final Rectangle lastLoc = path.get(lastLocPos).resolveBounds(
					viewer, cachedFirstBounds.getSize());

			if (cachedFirstBounds == null || lastLoc == null) {
				// there is no possible interpolation for a non-defined start or
				// end point
				throw new IllegalArgumentException(
						"Start or end Localizer can not be found or need interpolation in this animated element: "
								+ model);
			}

			// We assert the start and end point to be defined
			int i = 1;

			interpolateLocalizers: while (i < lastLocPos) {
				Localizer currentLocalizer = path.get(i);
				if (!currentLocalizer.needsInterpolation()) {
					currentLocalizer.resolveBounds(viewer, cachedFirstBounds
							.getSize());
					i++;
					continue interpolateLocalizers;
				}

				final int nextDefIndex = getIndexOfNextNotInterpolated(i + 1);

				// build interpolation delta data for this range of Localizers
				Localizer base = path.get(i - 1);
				final Rectangle start = base.getActualBounds();
				final Rectangle end = path.get(nextDefIndex).getActualBounds();
				final double scale = 1.0 / (nextDefIndex - (i - 1));
				final Dimension locDelta = start.getLocation().getDifference(
						end.getLocation()).scale(scale);
				final Dimension sizeDelta = start.getSize().getDifference(
						end.getSize()).scale(scale);

				// optimization: reusing the deltas till the next
				// non-interpolated Localizer is reached
				while (i < nextDefIndex) {
					currentLocalizer.resolveBounds(viewer, cachedFirstBounds
							.getSize());
					currentLocalizer.interpolateBounds(base, locDelta,
							sizeDelta);
					// proceed to next possibly to be interpolated Localizer
					i++;
					base = currentLocalizer;
					currentLocalizer = path.get(i);
				}
			}
		}

		/**
		 * @param start
		 * @return
		 */
		private int getIndexOfNextNotInterpolated(final int start) {
			for (int i = start; i <= path.size() - 1; i++) {
				if (!path.get(i).needsInterpolation()) {
					return i;
				}
			}
			// return that all all Localizers after start are interpolated
			return -1;
		}

		/**
		 * @param i
		 */
		public void prepareStep(final int i) {
			if (canPerform) {
				// FIXME Eigentlich wird das nur einmal fuer einen layer
				// aufgerufen und nicht fuer jede Figure einzeln!! Vielleicht
				// waere eine Hilfsklasse AnimatedViewer/Layer hilfreich, die
				// mehrfache revalidates und hookNeedsCapture (oder ist das
				// unntig?) verhindert oder managt?
				// Hoffentlich funktioniert das so
				// FIXME Test: Eigener Animator fr jedes Element, klappt NOCH
				// nicht... Animation anpassen, dass mehrere updateManager
				// untersttzt werden! Scheint zu klappen...
				layer.revalidate();
				// little fix for undo: when retrieving the first location, use
				// the cached rectangle, so that undo can find the original
				// location of the figure
				// FIXME Auf einem verschobenem Viewportausschnitt stimmen die
				// Locations nicht! Vielleicht laeuft Verschiebung der Location
				// durch den
				// ViewPort und Scaling falsch herum ab. Scaling muss nachher
				// oder vorher um den entsprechenden Anteil korrigiert werden
				final Rectangle newBounds;
				if (getIndexOfNextNotInterpolated(0) != i) {
					newBounds = path.get(i).getActualBounds();
				} else {
					newBounds = cachedFirstBounds.getCopy();
					newBounds.setLocation(newBounds.getLocation().scale(
							currentZoomLevel / initialZoomLevel));
				}
				final Point scaledLocation = newBounds.getLocation().scale(
						1 / currentZoomLevel);
				newBounds.setLocation(scaledLocation);
				figure.setBounds(newBounds);
				// This is a fix, so that Animation.doRun() need not to validate
				// (which presumably calls hookNeedCapture via Animator's
				// postLayout)
				// which would layout figure to its old position!
				MultipleAnimation.hookNeedsCapture(layer, animator);
			}
		}

		/**
		 * 
		 */
		public void animationDone() {
			if (canPerform) {
				originalParent.add(figure);
				figure.setBounds(originalBounds);
				layer.removeLayoutListener(animator);
				canPerform = false;
			}
			viewer = null;
			figure = null;
			originalParent = null;
			originalBounds = null;
			layer = null;
			currentZoomLevel = 0;
		}

		/**
		 * This method is used by {@link AnimatingCommand} to find the
		 * {@link MultiViewerPageViewer} in which it should animate itself. This
		 * must been hosted in an {@link AbstractMultiViewerPage}.
		 * 
		 * @param model
		 *            The model whose hosting viewer is been looked for
		 * @return The unique viewer having the model as its contents.
		 *         <code>null</code> if not such a unique viewer exists.
		 */
		protected static EditPartViewer findViewerShowing(final EObject model) {
			final ArrayList<EditPartViewer> viewers = new ArrayList<EditPartViewer>();
			final IWorkbenchPage activePage = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage();
			final IViewReference[] viewRefs = activePage.getViewReferences();
			for (final IViewReference viewRef : viewRefs) {
				final IViewPart view = viewRef.getView(false);
				if (view instanceof AbstractPageBookView) {
					final IPage page = ((AbstractPageBookView) view)
							.getCurrentPage();
					if (page instanceof AbstractMultiViewerPage) {
						final EditPartViewer viewer = ((AbstractMultiViewerPage) page)
								.getViewer(model);
						if (viewer != null) {
							viewers.add(viewer);
						}
					}
				}
			}
			// return viewer only if it is the only one showing the model
			// TODO Knnte auch mit Array von Viewers funktionieren, ist aber
			// noch nicht ntig
			if (viewers.size() == 1) {
				return viewers.get(0);
			}
			return null;
		}

	}

	/**
	 * Data class describing a location (possibly indirect through a model
	 * element) and the size (factor) an {@link AnimatedElement} should be set
	 * to when reaching this {@link Localizer} while following its path.
	 * 
	 * @author Tony Modica
	 */
	public static class Localizer {

		/**
		 * A target model element specifying a location which will be retrieved
		 * in a viewer context by it's edit parts.
		 */
		private EObject targetModel;

		/**
		 * The absolute location to be taken into account when resolving the
		 * bounds of this Localizer.
		 */
		private Point absoluteLocation;

		/**
		 * The relative size factor to be taken into account when resolving the
		 * bounds of this Localizer.
		 */
		private double sizeFactor;

		/**
		 * The absolute size to be taken into account when resolving the bounds
		 * of this Localizer.
		 */
		private Dimension absoluteSize;

		/**
		 * Transient data fields used to store resolved and interpolated
		 * results.
		 */
		public Point actualLocation;
		public Dimension actualSize;

		/**
		 * Convenience constructor for a placeholder Localizer whose data is to
		 * be interpolated completely.
		 * 
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Localizer() {
			this(null, null);
		}

		/**
		 * <p>
		 * Convenience constructor for a Localizer whose relative size factor
		 * will be 1 or determined by a passed {@link Rectangle}. If the size
		 * of this {@link Localizer} is meant to be interpolated a different
		 * constructor should be used!
		 * </p>
		 * 
		 * <p>
		 * A valid argument has to be a {@link Rectangle} (its {@link Dimension}
		 * will be considered as absolute size specification!), {@link EObject},
		 * {@link GraphicalEditPart}, {@link Point}, or <code>null</code>.
		 * </p>
		 * 
		 * @param object
		 *            Object used to resolve location (and possibly size) of
		 *            this Localizer in viewers.
		 * 
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Localizer(final Object object) {
			if (object instanceof Rectangle) {
				final Rectangle bounds = (Rectangle) object;
				absoluteLocation = bounds.getLocation();
				absoluteSize = bounds.getSize();
			} else if (object instanceof EObject) {
				targetModel = (EObject) object;
				sizeFactor = 1;
			} else if (object instanceof GraphicalEditPart) {
				targetModel = (EObject) ((GraphicalEditPart) object).getModel();
				sizeFactor = 1;
			} else if (object instanceof Point) {
				absoluteLocation = (Point) object;
				sizeFactor = 1;
			} else if (object == null) {
				sizeFactor = 1;
			} else {
				throw new IllegalArgumentException(
						"Localizers must be defined by EObject, GraphicalEditPart, Point or Rectangle!");
			}
		}

		/**
		 * Main constructor for {@link Localizer}s with an absolute size
		 * specification. If passed object and/or size are <code>null</code>
		 * this {@link Localizer} will need to be interpolated.
		 * 
		 * @param object
		 *            Object used to resolve location of this Localizer in
		 *            viewers. Set to <code>null</code> if location is meant
		 *            to be interpolated.
		 * @param size
		 *            {@link Dimension} used to resolve size of this Localizer
		 *            in viewers. Set to <code>null</code> if size is meant to
		 *            be interpolated.
		 * 
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Localizer(final Object object, final Dimension size) {
			if (object instanceof EObject) {
				targetModel = (EObject) object;
			} else if (object instanceof GraphicalEditPart) {
				targetModel = (EObject) ((GraphicalEditPart) object).getModel();
			} else if (object instanceof Point) {
				absoluteLocation = (Point) object;
			} else if (object == null) {
			} else {
				throw new IllegalArgumentException(
						"Localizer must be defined by EObject, GraphicalEditPart or Point!");
			}
			absoluteSize = size;
			sizeFactor = -1;
		}

		/**
		 * Main constructor for {@link Localizer}s with a relative size
		 * specification. If the passed object is <code>null</code> this
		 * {@link Localizer} will need to be interpolated.
		 * 
		 * @param object
		 *            Object used to resolve location of this Localizer in
		 *            viewers. Set to <code>null</code> if location is meant
		 *            to be interpolated.
		 * @param sizeFactor
		 *            double scaling factor used to resolve size of this
		 *            Localizer in viewers. Set to -1 if size is meant to be
		 *            interpolated.
		 * 
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Localizer(final Object object, final double sizeFactor) {
			this.sizeFactor = sizeFactor;
			if (object instanceof EObject) {
				targetModel = (EObject) object;
			} else if (object instanceof GraphicalEditPart) {
				targetModel = (EObject) ((GraphicalEditPart) object).getModel();
			} else if (object instanceof Point) {
				absoluteLocation = (Point) object;
			} else if (object == null) {
			} else {
				throw new IllegalArgumentException(
						"Localizer must be defined by EObject, GraphicalEditPart or Point!");
			}
		}

		/**
		 * This should be called only after calling
		 * {@link #resolveBounds(EditPartViewer, IFigure)} and, if needed,
		 * {@link #interpolateBounds(animation.AnimatingCommand.Localizer, Dimension, Dimension)}.
		 * 
		 * @return A {@link Rectangle} created from {@link #actualLocation} and
		 *         {@link #actualSize} of thie Localizer if they are both not
		 *         <code>null</code>, otherwise <code>null</code>. This
		 *         could happen if the target model of this {@link Localizer}
		 *         can not be found in a viewer.
		 * 
		 * @see #resolveBounds(EditPartViewer, IFigure)
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Rectangle getActualBounds() {
			if (actualLocation != null && actualSize != null) {
				return new Rectangle(actualLocation, actualSize);
			}
			// May happen if no editPart for targetModel could be found in
			// viewer
			return null;
		}

		/**
		 * <p>
		 * Tries to resolve the bounds, i.e. location and size, the figure
		 * should be set to when reaching this Localizer.
		 * </p>
		 * 
		 * <p>
		 * This method should be called always when considering a Localizer in a
		 * viewer context. Afterwards it may still need to call
		 * {@link #interpolateBounds(animation.AnimatingCommand.Localizer, Dimension, Dimension)}
		 * which can be checked with {@link #needsInterpolation()};
		 * 
		 * @param viewer
		 * @param figure
		 * @return The resolved bounds that {@link #getActualBounds()} would
		 *         return, for convenience. May be <code>null</code> if no
		 *         edit part can be found for the target model or if
		 *         interpolation is needed.
		 * 
		 * @see #needsInterpolation()
		 * @see #interpolateBounds(animation.AnimatingCommand.Localizer,
		 *      Dimension, Dimension)
		 */
		public Rectangle resolveBounds(final EditPartViewer viewer,
				final Dimension initialDim) {
			Assert.isTrue(viewer != null && initialDim != null,
					"Localizer needs viewer and initial dimension to resolve!");
			resolveLocation(viewer);
			resolveSize(initialDim);
			return getActualBounds();
		}

		private void resolveLocation(final EditPartViewer viewer) {
			// if a model has been set as locating object find the figure of
			// it's editpart and translate the location to absolute
			if (targetModel != null) {
				final GraphicalEditPart targetEditPart = (GraphicalEditPart) viewer
						.getEditPartRegistry().get(targetModel);
				if (targetEditPart == null) {
					return;
				}
				final IFigure targetFigure = targetEditPart.getFigure();
				actualLocation = targetFigure.getBounds().getLocation();
				targetFigure.translateToAbsolute(actualLocation);
			} else {
				actualLocation = absoluteLocation;
			}
		}

		private void resolveSize(final Dimension initialDim) {
			if (sizeFactor >= 0) {
				actualSize = initialDim.getScaled(sizeFactor);
			} else {
				actualSize = absoluteSize;
			}
		}

		/**
		 * Check if location or size have to be interpolated. This would require
		 * to call
		 * {@link #interpolateBounds(animation.AnimatingCommand.Localizer, Dimension, Dimension)}
		 * with additional data.
		 * 
		 * @return Whether location and/or size of this localizer would not be
		 *         resolved completely by
		 *         {@link #resolveBounds(EditPartViewer, IFigure)}.
		 */
		public boolean needsInterpolation() {
			final boolean locationInterpolated = (absoluteLocation == null && targetModel == null);
			final boolean sizeInterpolated = (absoluteSize == null && sizeFactor == -1);
			return locationInterpolated || sizeInterpolated;
		}

		public Rectangle interpolateBounds(final Localizer base,
				final Dimension locDelta, final Dimension sizeDelta) {
			if (actualLocation == null) {
				// shift base location by location delta
				actualLocation = base.actualLocation.getTranslated(locDelta);
			}

			if (actualSize == null) {
				// change size by size delta
				actualSize = base.actualSize.getExpanded(sizeDelta);
			}

			return getActualBounds();
		}
	}
}
