/**
 * 
 */
package roneditor.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.commands.CreateHLTokenCommand;
import roneditor.commands.DeleteTokenCommand;
import roneditor.dialogs.RuleSimulator;
import roneditor.dialogs.chooseObjNet.AbstractChooseObjNetDialog;
import roneditor.dialogs.chooseObjNet.ChooseObjNetDialog;
import roneditor.dialogs.chooseRule.AbstractChooseRuleDialog;
import roneditor.dialogs.chooseRule.ChooseRuleDialog;
import roneditor.dialogs.fire.FireDialog;
import roneditor.dialogs.match.AbstractMatchDialog;
import roneditor.dialogs.match.MatchDialog;
import roneditor.objectnetview.commands.AutoLayoutCommand;
import roneditor.objectnetview.commands.DeleteBendpointCommand;
import roneditor.objectnetview.commands.FireONTransitionCommand;
import roneditor.objectnetview.edit.parts.ONArcEditPart;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import roneditor.ronview.commands.CopyHLTokenCommand;
import roneditor.ronview.commands.MergeObjectNetsCommand;
import roneditor.ronview.commands.MoveHLTokenCommand;
import roneditor.ronview.commands.SplitObjectNetCommand;
import roneditor.ronview.edit.parts.HLTransitionEditPart;
import ronmodel.BendPoint;
import ronmodel.Codomain;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/** 
 * Action zum Schalten von ONTransitions und HLTransitions
 * 
 * @author Mario
 */
public class FireTransitionAction extends SelectionAction {

	
	static public String ID = "Fire_Transition_Action";
	
	/**
	 * Initializiert eine neu erstellte FireTransitionAction. Die Action is danach ausfhrbereit.
	 * @param part IWorkbenchPart in dem die Action ausgefhrt werden soll 
	 * 
	 */
	public FireTransitionAction(IWorkbenchPart part) {
		super(part);
		init();
	}
	
	/**
	 * Fhrt die Action aus. Dabei wird entsprechend der angewhlten Transition, das Schaltenverhalten in einem Command verpackt 
	 * und dieses auf den beim erstellen dieser Action angebenen WorkbenchPart ausgefhrt.
	 * 
	 */
	public void run() {
		Command command = null;
		
		// selektiertes Objekt holen (das Erste) 
		Object selection =
			((StructuredSelection) getSelection()).getFirstElement();
		
		// command ber Unterfunktionen entsprechend der ausgewhlten Transition setzen
		if (selection instanceof ONTransitionEditPart) {
			command = getONTransitionFireCommand(((ONTransitionEditPart)selection).getONTransition());
		} else if (selection instanceof HLTransitionEditPart) {
			HLTransition hlTransition = ((HLTransitionEditPart)selection).getHLTransition();

			switch (hlTransition.getKind().getValue()) {
			case HLTransitionKind.FIRE :
				command = getHLTransitionFireONTransitionCommand(hlTransition);
				break;
			case HLTransitionKind.SPLIT :
				command = getHLTransitionFireSplitCommand(hlTransition);
				break;
			case HLTransitionKind.STANDARD :
				command = getHLTransitionFireStandardCommand(hlTransition);
				break;
			case HLTransitionKind.APPLYRULE :
				command = getHLTransitionFireApplyRuleCommand(hlTransition);
				break;
			default : command = null;				
			}			
		}		
		// Command auf WorkbenchPart ausfhren
		execute(command);
	}
		
	/**
	 * Es werden von jeder Vorbereichsstelle soviele Token weggenommen, wie das Gewicht der entsprechenden Eingangskante ist.
	 * Und auf die Nachbereichsstellen soviele Token raufgetan, wie das Gewicht der entsprechenden Ausgangskante ist.
	 * 
	 * @param onTransition ONTransition die Schalten soll.
	 * @return Command welches die Modelnderungen beim Feuern eine ONTransition umsetzt
	 */
	private Command getONTransitionFireCommand(ONTransition onTransition){		
		FireONTransitionCommand fireCommand = new FireONTransitionCommand();			
		fireCommand.setONTransition(onTransition);
		return fireCommand; 	
	}

	
	/** 
	 * Von der Vorbereichsstelle wird ein Object-Netz mittels einem Dialog ausgewhlt.
	 * In diesem Object-Netz wird mit dem gleichen Dialog auch die zu schalten ONTransition ausgewhlt.
	 * 
	 * @param hlTransition HLTransition die Schalten soll
	 * @return Command welches die Modelnderungen beim Feuern eine ONTransition umsetzt 
	 * und das ObjectNetz von der vorberreichsstelle zur Nachbereichsstelle bewegt.
	 */
	private Command getHLTransitionFireONTransitionCommand(HLTransition hlTransition){
	
		
		if (hlTransition == null || hlTransition.getIncoming().size() != 1) return null;
		final HLPlace prePlace = (HLPlace)((HLArc)hlTransition.getIncoming().get(0)).getSource();
		

	
		if (prePlace.getTokens().isEmpty()) {
			// nothing to do
			return null;
		}
	
		// Fire Dialog ffnen
		final FireDialog dialog = new FireDialog(prePlace.getTokens());
		dialog.setText("Whle ObjectNet von HLPlace "+prePlace.getName());
		dialog.open();
		if (dialog.isCanceled()) {
			return null;
		}
	
		// fetch information from the dialog
		final ONTransition choosenONTransition = dialog.getTransitionToFire();
	
		CompoundCommand cCommand = new CompoundCommand();
		
		FireONTransitionCommand fireCommand = new FireONTransitionCommand();
		fireCommand.setONTransition(choosenONTransition);
		cCommand.add(fireCommand);
		
		for (Object o: hlTransition.getOutgoing()){			
			CopyHLTokenCommand copyCommand = new CopyHLTokenCommand();
			copyCommand.setToken(choosenONTransition.getObjectNet());
			copyCommand.setNewHLPlace((HLPlace)((HLArc)o).getTarget());
			cCommand.add(copyCommand);			
		}
		
		DeleteTokenCommand deleteToken = new DeleteTokenCommand(choosenONTransition.getObjectNet());
		cCommand.add(deleteToken);
		
		return cCommand;
	}
	
	/** 
	 * Gibt es mehrere NET-Stellen im Vorbereich, dann werden von allen Vorbereichsstellen jeweils 
	 * ein Object-Netz mittels einem Dialog ausgewhlt. Diese Object-Netze werden dann zu einem verschmolzen und automatisch gelayoutet und
	 * im MatchDialog mit allen Regeln von der RULE-Stelle im Vorbereich zur Regelanwendung zur Verfgung gestellt.
	 * Gibt es nur eine NET-Stelle im Vorbereich, dann werden alle Object-Netze dieser Stelle und alle Regel von der 
	 * RULE-Stelle im Vorbereich im MatchDialog zur Regelanwendung zur Verfgung gestellt.
	 * Das MergeCommand(nur wenn ntig), das Command, welches die Regelanwendung ausfhrt, das Command, 
	 * welches das AuotLayout durchfhrt, das Command, welches das Verschieben bzw. eventuelles Lschen umsetzt, und 
	 * Kopieren des resultierenden Object-Netzes werden in einem CompoundCommand verpackt und zurckgeben. 
	 * 
	 * @param hlTransition HLTransition die Schalten soll
	 * @return Command welches das Anwenden einer Regel im Model umsetzt
	 */
	private Command getHLTransitionFireApplyRuleCommand(HLTransition hlTransition){

		// Auswahl der Regel und Object-Netze im Vorbereich der HLTransition
		final ArrayList<Rule> rules = new ArrayList<Rule>();
		final ArrayList<ObjectNet> choosenObjectNets = new ArrayList<ObjectNet>();
		
		HLPlace preRulePlace  = null;
		final ArrayList<HLPlace> preObjectNetPlaces  = new ArrayList<HLPlace>();
		
		for (Object obj: hlTransition.getIncoming()) {
			HLArc hlArc = (HLArc)obj;
			HLPlace hlPlace = (HLPlace)hlArc.getSource();
			if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
				preObjectNetPlaces.add(hlPlace);
			} else if (hlPlace.getType().equals(HLPlaceType.RULE_LITERAL)) {
				// darf nur eine sein
				preRulePlace= hlPlace;
			}
		}		

		Boolean mergedNeeded = false;
		// wenn mehr als eine NET-Stelle im Vorbereich, dann muss Auswahl wegen Merge vorher stattfinden
		if (preObjectNetPlaces.size() > 1) {
			mergedNeeded = true;
			// alle ObjectNetze im Vorbereich der HLTransition durchgehen		
			for (HLPlace hlPlace: preObjectNetPlaces) {
				if (hlPlace.getTokens().size() > 1) {
					// bei mehr als einem Token, AuswahlDialog ffnen
					final AbstractChooseObjNetDialog dialog = new ChooseObjNetDialog(hlPlace.getTokens());
					dialog.setText("Whle ObjectNet von HLPlace "+hlPlace.getName());
					dialog.open();
					if (dialog.isCanceled()) {
						return null;
					}			
					choosenObjectNets.add(dialog.getSelectedObjectNet());					
				} else if (hlPlace.getTokens().size() == 1) {
					// nur ein Token
					choosenObjectNets.addAll(hlPlace.getTokens());	
				}	
			}
		} else if (preObjectNetPlaces.size() == 1) {
			// Auswahl des Object-Netzes erst im MatchDialog, da kein merge/AutoLayout vorher ntig
			choosenObjectNets.addAll(preObjectNetPlaces.get(0).getTokens());
		} else {
			return null;
		}

		if (preRulePlace != null) {
			
			rules.addAll(preRulePlace.getTokens());
		} else {
			return null;
		}
		

		if (choosenObjectNets.isEmpty() || rules.isEmpty()) {
			// mind. eine Regel und ein ObjectNetz ntig
			// sollte eigentlich durch calculateEnabled() schon ausgeschlossen sein
			return null;
		}
		
		AbstractMatchDialog dialog = null;
		if (mergedNeeded) {
			// Merge fr MatchDialog
			MergeObjectNetsCommand mergeCommand = new MergeObjectNetsCommand();
			mergeCommand.setObjNets(choosenObjectNets);
	
			// Autolayout damit es etwas besser im MatchDialog aussieht
			AutoLayoutCommand layoutCommand = new AutoLayoutCommand();
			layoutCommand.setObjectNet(choosenObjectNets.get(0));
			
			mergeCommand.execute();
			layoutCommand.execute();

			// MatchDialog braucht eine List von Object-Netzen
			ArrayList<ObjectNet> objNetTemp = new ArrayList<ObjectNet>();
			objNetTemp.add(choosenObjectNets.get(0));
			dialog = new MatchDialog(rules,objNetTemp);
			// MatchDialog ffnen
			dialog.open();
			// das AutoLayout und Mergen vorerst wieder Rckgngig machen
			// die im MatchDialog erstellte Kopie ist gemergt/autolayoutet
			// das Orginal soll unverndert bleiben
			layoutCommand.undo();
			mergeCommand.undo();
		} else {
			dialog = new MatchDialog(rules,choosenObjectNets);
			// MatchDialog ffnen
			dialog.open();
		}
				
		
		
		if (dialog.isCanceled()) {
			return null;
		}

		// fetch information from the dialog
		final Rule rule = dialog.getCurrentRule();
		final Codomain objectNet = dialog.getObjectNetCopy();
		final ObjectNet originalObjectNet= dialog.getObjectNet();
		final HLPlace parentPlace = dialog.getHLPlace();
		final HashMap<ONNode, ONNode> match = dialog.getMatchToApply();
		
		
		// apply the selected rule to the selected object net
		(new RuleSimulator()).applyRule(match, rule, objectNet);
		
		
		CompoundCommand cCommand = new CompoundCommand();
		
		// Copy auf orginalStelle einfgen
		CreateHLTokenCommand createCommand = new CreateHLTokenCommand(parentPlace,objectNet);
		createCommand.setName(objectNet.getName());
		cCommand.add(createCommand);
		
		// alle orignialen ObjektNetze lschen
		if (preObjectNetPlaces.size() > 1) { 
			for (ObjectNet objNet : choosenObjectNets) {
				DeleteTokenCommand deleteCommand = new DeleteTokenCommand(objNet);
				cCommand.add(deleteCommand);
			}
		} else {
			DeleteTokenCommand deleteCommand = new DeleteTokenCommand(originalObjectNet);
			cCommand.add(deleteCommand);
		}
		
		
		// Nachbereichs NetStellen und Rulestellen finden
		final ArrayList<HLPlace> postRulePlaces = new ArrayList<HLPlace>();
		final ArrayList<HLPlace> postObjectNetPlaces = new ArrayList<HLPlace>();
		
		for (Object obj: hlTransition.getOutgoing()) {
			HLArc hlArc = (HLArc)obj;
			HLPlace hlPlace = (HLPlace)hlArc.getTarget();
			if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
				postObjectNetPlaces.add(hlPlace);
			} else if (hlPlace.getType().equals(HLPlaceType.RULE_LITERAL)) {
				postRulePlaces.add(hlPlace);
			}
		}
		
		// Object-Netz Stellen behandeln
		
		// wenn keine Stellen im Nachbereich sind, den Token im Vorbereich einfach lschen
		if (postObjectNetPlaces.size() == 0) {
			DeleteTokenCommand deleteToken = new DeleteTokenCommand(objectNet);
			cCommand.add(deleteToken);
		} else {
			MoveHLTokenCommand moveCommand = new MoveHLTokenCommand();
			moveCommand.setToken(objectNet);
			moveCommand.setNewHLPlace(postObjectNetPlaces.get(0));
			cCommand.add(moveCommand);
			
			// wenn es mehrere ausgehenden Kanten gibt, muss das Netz noch auf alle kopiert werden
			for (int i=1; i < postObjectNetPlaces.size(); i++) {
				CopyHLTokenCommand copyCommand = new CopyHLTokenCommand();
				copyCommand.setToken(objectNet);			
				copyCommand.setNewHLPlace(postObjectNetPlaces.get(i));
				cCommand.add(copyCommand);
			}		
		}
		
		
		// Rule Stellen behandeln
		
		// wenn keine Stellen im Nachbereich sind, den Token im Vorbereich einfach lschen
		if (postRulePlaces.size() == 0) {
			DeleteTokenCommand deleteToken = new DeleteTokenCommand(rule);
			cCommand.add(deleteToken);
		} else {
			MoveHLTokenCommand moveCommand = new MoveHLTokenCommand();
			moveCommand.setToken(rule);
			moveCommand.setNewHLPlace(postRulePlaces.get(0));
			cCommand.add(moveCommand);
			
			// wenn es mehrere ausgehenden Kanten gibt, muss das Netz noch auf alle kopiert werden
			for (int i=1; i < postRulePlaces.size(); i++) {
				CopyHLTokenCommand copyCommand = new CopyHLTokenCommand();
				copyCommand.setToken(rule);			
				copyCommand.setNewHLPlace(postRulePlaces.get(i));
				cCommand.add(copyCommand);
			}		
		}
		
		return cCommand;
		
	}
	

	/**
	 * Von der Vorbereichsstelle wird ein Object-Netz genommen.
	 * Dieses Object-Netz wird auf die eine Nachbereichs-NET-Stelle verschoben und dann in entsprechend viele
	 * zusammenhngende Object-Netze getrennt.
	 * 
	 * @param hlTransition HLTransition die Schalten soll
	 * @return Command welches das Schalten der Selectierten Split-Transition im Model umsetzt
	 */
	private Command getHLTransitionFireSplitCommand(HLTransition hlTransition){
		ObjectNet choosenObjectNet = null;


		// Alle Object-Netze im Vorbereich der Transition finden
		// nur eine Vorbereichsstelle erlaubt
		if (hlTransition.getIncoming().size()!=1) return null;
		HLArc hlArc = (HLArc)hlTransition.getIncoming().get(0);
		HLPlace hlPlace = (HLPlace)hlArc.getSource();
		if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
			if (hlPlace.getTokens().size() > 1) {
				// Auswahldialog ffnen
				final ChooseObjNetDialog dialog = new ChooseObjNetDialog(hlPlace.getTokens());
				dialog.setText("Whle ObjectNet von HLPlace "+hlPlace.getName());
				dialog.open();
				if (dialog.isCanceled()) {
					return null;
				}
				choosenObjectNet = dialog.getSelectedObjectNet();
			} else if (hlPlace.getTokens().size() == 1){
				// nur ein Token, Auswahl Dialog nicht ntig
				choosenObjectNet = (ObjectNet)hlPlace.getTokens().get(0);
			}
		}
		
		
		if (choosenObjectNet == null) {
			// Erzeugen von Object-Netzen aus dem Nichts, nicht mglich
			return null;
		}
		

		CompoundCommand cCommand = new CompoundCommand();
		// choosenObjectNet auf neue Stelle bewegen
		MoveHLTokenCommand moveCommand = new MoveHLTokenCommand();
		moveCommand.setToken(choosenObjectNet);
		moveCommand.setNewHLPlace((HLPlace)((HLArc)hlTransition.getOutgoing().get(0)).getTarget());
		cCommand.add(moveCommand);
		
		// choosenObjectNet aufsplitten		
		SplitObjectNetCommand splitCommand = new SplitObjectNetCommand();
		splitCommand.setObjNet(choosenObjectNet);
		cCommand.add(splitCommand);
		return cCommand;
	}
	
	/**
	 * Von jeder Vorbereichsstelle wird ein Token genommen.
	 * Object-Netze werden dann wenn ntig zusammengefgt und auf alle Nachbereichs-NET-Stellen kopiert.
	 * Es darf nur eine RULE-Stelle im Vorbereich geben und diese wird auf alle Nachbereichs-RULE_Stellen kopiert.
	 * 
	 * @param hlTransition HLTransition die Schalten soll
	 * @return Command welches das Schalten der Selectierten Standard-Transition im Model umsetzt
	 */
	private Command getHLTransitionFireStandardCommand(HLTransition hlTransition){

		// Auswahl der Regel und ObjectNetze im Vorbereich der HLTransition
		Rule choosenRule = null;
		final ArrayList<ObjectNet> choosenObjectNets = new ArrayList<ObjectNet>();


		// alle Regeln und ObjectNetze im Vorbereich der HLTransition durchgehen
		for (Object obj: hlTransition.getIncoming()) {
			HLArc hlArc = (HLArc)obj;
			HLPlace hlPlace = (HLPlace)hlArc.getSource();
			if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
				if (hlPlace.getTokens().size() > 1) {
					// bei mehr als einem Token, AuswahlDialog ffnen
					final AbstractChooseObjNetDialog dialog = new ChooseObjNetDialog(hlPlace.getTokens());
					dialog.setText("Whle ObjectNet von HLPlace "+hlPlace.getName());
					dialog.open();
					if (dialog.isCanceled()) {
						return null;
					}			
					choosenObjectNets.add(dialog.getSelectedObjectNet());					
				} else if (hlPlace.getTokens().size() == 1) {
					// nur ein Token
					choosenObjectNets.addAll(hlPlace.getTokens());	
				}				
			} else if (hlPlace.getType().equals(HLPlaceType.RULE_LITERAL)) {
				if (hlPlace.getTokens().size() > 1) {
					// bei mehr als einem Token, AuswahlDialog ffnen
					final AbstractChooseRuleDialog dialog = new ChooseRuleDialog(hlPlace.getTokens());
					dialog.setText("Whle Rule von HLPlace "+hlPlace.getName());
					dialog.open();
					if (dialog.isCanceled()) {
						return null;
					}	
					choosenRule = dialog.getSelectedRule();
				} else if (hlPlace.getTokens().size() == 1) {
					// nur ein Token
					choosenRule = (Rule)hlPlace.getTokens().get(0);	
				}
				
			}
		}
		

		if (choosenObjectNets.isEmpty() && choosenRule == null) {
			// Erzeugen von ObjectNetzen/Rules auf den Nachbereichsstellen aus dem Nichts, nicht mglich
			return null;
		}

		CompoundCommand cCommand = new CompoundCommand();	
		
		// alle ausgewhlten ObjectNetze aus dem Vorbereich mergen(zusammenfgen), wenn es mehrere sind
		if (choosenObjectNets.size() > 1) {
			MergeObjectNetsCommand mergeCommand = new MergeObjectNetsCommand();
			mergeCommand.setObjNets(choosenObjectNets);
			cCommand.add(mergeCommand);
		}
		
		// Nachbereichs Net-Stellen und Rule-Stellen finden
		final ArrayList<HLPlace> postRulePlaces = new ArrayList<HLPlace>();
		final ArrayList<HLPlace> postObjectNetPlaces = new ArrayList<HLPlace>();

		
		for (Object obj: hlTransition.getOutgoing()) {
			HLArc hlArc = (HLArc)obj;
			HLPlace hlPlace = (HLPlace)hlArc.getTarget();
			if (hlPlace.getType().equals(HLPlaceType.NET_LITERAL)) {
				postObjectNetPlaces.add(hlPlace);
			} else if (hlPlace.getType().equals(HLPlaceType.RULE_LITERAL)) {
				postRulePlaces.add(hlPlace);
			}
		}
		
		// ObjectNetz Stellen behandeln
		
		// ObjectNet im Vorbereich, sonst nichts machen
		if (choosenObjectNets.size() > 0) {		
			
			// wenn keine Rule-Stellen im Nachbereich sind, den Token im Vorbereich einfach lschen
			if (postObjectNetPlaces.size() == 0) {
				DeleteTokenCommand deleteToken = new DeleteTokenCommand(choosenObjectNets.get(0));
				cCommand.add(deleteToken);
			} else {
				// das transformierte ON auf die erste Nachbereichsstelle verschieben
				MoveHLTokenCommand moveCommand = new MoveHLTokenCommand();
				moveCommand.setToken(choosenObjectNets.get(0));
				moveCommand.setNewHLPlace(postObjectNetPlaces.get(0));
				cCommand.add(moveCommand);
				
				// wenn es mehrere ausgehenden Kanten gibt, muss das Netz noch auf alle Kopiert werden
				for (int i=1; i < postObjectNetPlaces.size(); i++) {
					CopyHLTokenCommand copyCommand = new CopyHLTokenCommand();
					copyCommand.setToken(choosenObjectNets.get(0));			
					copyCommand.setNewHLPlace(postObjectNetPlaces.get(i));
					cCommand.add(copyCommand);
				}		
			}
		}
		
		
		// Rule Stellen behandeln
		// Regel im Vorbereich? sonst nichts machen
		if (choosenRule != null) {
			if (postRulePlaces.size() == 0) {
				// wenn keine Rule-Stellen im Nachbereich sind, den Token im Vorbereich einfach lschen
				DeleteTokenCommand deleteToken = new DeleteTokenCommand(choosenRule);
				cCommand.add(deleteToken);
			} else {
				// die unvernderte Rule auf die erste Nachbereichsstelle verschieben
				MoveHLTokenCommand moveCommand = new MoveHLTokenCommand();
				moveCommand.setToken(choosenRule);
				moveCommand.setNewHLPlace(postRulePlaces.get(0));
				cCommand.add(moveCommand);
				
				// wenn es mehrere ausgehenden Kanten gibt, muss die Rule noch auf alle kopiert werden
				for (int i=1; i < postRulePlaces.size(); i++) {
					CopyHLTokenCommand copyCommand = new CopyHLTokenCommand();
					copyCommand.setToken(choosenRule);			
					copyCommand.setNewHLPlace(postRulePlaces.get(i));
					cCommand.add(copyCommand);
				}		
			}
		}
		
		return cCommand;
	}
	
	

	/**
	 * Gibt true zurck wenn die private Funktion canPerformAction() dies tut.
	 * 
	 * @return true wenn canPerformAction()==true , andernfalls false
	 */
	protected boolean calculateEnabled() {
			return canPerformAction();
	}
	
	
	
	/**
	 * Gibt true zurck, wenn nur ein Object selektiert ist, dieses ein ONTransitionEditPart oder HLTransitionEditPart ist 
	 * und die jeweilige fireEnabled() Funktion true zurck gibt. Andernfalls wird false zurckgegeben.
	 * 	 
	 * @return true if action could be executed, otherwise false
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		
		List parts = getSelectedObjects();
		// nur Einzelselectionen erlaubt
		if (parts.size()==1){
			// die entsprechende fireEnabled Funktion des TransitionsEditparts benutzt
			if (parts.get(0) instanceof ONTransitionEditPart) {	
				return ((ONTransitionEditPart)parts.get(0)).fireEnabled();			
			} else if (parts.get(0) instanceof HLTransitionEditPart) {
				return ((HLTransitionEditPart)parts.get(0)).fireEnabled();				
			}
		} 
		return false;
	}

	/**
	 * Initialisert die Action. Name, Description und ID werden gesetzt.
	 */
	protected void init() {
		super.init();
		setDescription("Fire Transition Action");
		setText("Fire Transition");
		setId(ID);
	}
	
}
