package roneditor.dialogs;

import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;

import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.impl.RonmodelFactoryImpl;
import agg.attribute.AttrType;
import agg.attribute.handler.AttrHandler;
import agg.attribute.handler.impl.javaExpr.JexHandler;
import agg.xt_basis.Arc;
import agg.xt_basis.DefaultGraTraImpl;
import agg.xt_basis.GraGra;
import agg.xt_basis.GraTra;
import agg.xt_basis.Graph;
import agg.xt_basis.GraphObject;
import agg.xt_basis.Match;
import agg.xt_basis.Node;
import agg.xt_basis.OrdinaryMorphism;
import agg.xt_basis.Type;
import agg.xt_basis.TypeSet;


/**
 * This class uses the graph transformation engine AGG to transform an object net
 * according to an object net rule.
 * 
 * @author enrico
 *
 */
public class RuleSimulator {
	private TypeSet typeSet = new TypeSet();
	private ObjectNet net;
	private Graph graph;
	
	private RonmodelFactory ronFactory;
	
	private HashMap<ONNode,Node> net2graphNodeMap = new HashMap<ONNode,Node>();
	private HashMap<ONArc,Arc> net2graphArcMap = new HashMap<ONArc,Arc>();
	
	private Vector<ONNode> createdNodes = new Vector<ONNode>();
	private Vector<ONNode> deletedNodes = new Vector<ONNode>();
	
	private Vector<EObject> createdArcs = new Vector<EObject>();
	private Vector<EObject> deletedArcs = new Vector<EObject>();
	
	private boolean executed = false;
	private boolean undone = false; 
	
	private final int TOKEN = 0;
	private final int CAPACITY = 0;
	private final int WEIGHT = 1;
	
	
	/**
	 * Creates a new RuleSimulator instance with precreated AGG type graph and Ronmodel factory.
	 *
	 */
	public RuleSimulator() {
		createTypeGraph();
		
		ronFactory = new RonmodelFactoryImpl();
	}

	/**
	 * Creates an AGG type graph with Place, Transition, ArcPT and ArcTP. 
	 *
	 */
	private void createTypeGraph() {
		AttrHandler handler = new JexHandler();
		
		// create place node type
		Type placeType = typeSet.createType(true);
		placeType.setStringRepr("Place");
		placeType.setAdditionalRepr(":RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:");
		AttrType attr = (AttrType) placeType.getAttrType();
		attr.addMember(handler, "String", "name");
		attr.addMember(handler, "int", "token");
		//attr.addMember(handler, "int", "capacity");
		
		//create transition node type
		Type transitionType = typeSet.createType(true);
		transitionType.setStringRepr("Transition");
		transitionType.setAdditionalRepr(":RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:");
		attr = (AttrType) transitionType.getAttrType();
		attr.addMember(handler, "String", "name");
		
		//create arcPT edge type
		Type arcPTType = typeSet.createType(true);
		arcPTType.setStringRepr("arcPT");
		arcPTType.setAdditionalRepr(":SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:");
		attr = (AttrType) arcPTType.getAttrType();
		attr.addMember(handler, "int", "weight");
		
		//create arcTP edge type
		Type arcTPType = typeSet.createType(true);
		arcTPType.setStringRepr("arcTP");
		arcTPType.setAdditionalRepr(":SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:");
		attr = (AttrType) arcTPType.getAttrType();
		attr.addMember(handler, "int", "weight");
	}
	
	/**
	 * Applies an object net rule to an object net.
	 * 
	 * @param onRule An object net rule that should be applied to an object net.
	 * @param net The object net the rule should be applied to.
	 * @return true, if the rule was successfully applied
	 */
	public boolean applyRule(Rule onRule, ObjectNet net) {
		return applyRule(null, onRule, net);
	}
	
	/**
	 * Applies an object net rule to an object net.
	 * 
	 * @param mapping A mapping between ONNodes in the LHS and the object net the rule should be applied to. 
	 * @param onRule An object net rule that should be applied to an object net.
	 * @param net The object net the rule should be applied to.
	 * @return true, if the rule was successfully applied
	 */
	public boolean applyRule(HashMap<ONNode, ONNode> mapping, Rule onRule, ObjectNet net) {
		if(!executed) {
			GraTra transformation = new DefaultGraTraImpl();
			GraGra grammar = new GraGra(typeSet);
			graph = grammar.getGraph();
			this.net = net;
			transformation.setGraGra(grammar);
			agg.xt_basis.Rule aggRule = grammar.createRule();
			net2graph(net, graph, net2graphNodeMap, net2graphArcMap, false);
			HashMap<ONNode,Node> lhsMap = onRule2aggRule(onRule, aggRule);
			
			Match aggMatch = transformation.createMatch(aggRule);
			onMapping2aggMapping(mapping, aggMatch, lhsMap, net2graphNodeMap);
			
			//grammar.save("test.ggx");

			boolean success = transformation.apply(aggRule);
			if (success) {
				updateObjectNet(net, graph, net2graphNodeMap, net2graphArcMap);
				executed = true;
			}
			return success;
		}
		
		return false;
	}

	
	public Vector<HashMap<ONNode,ONNode>> calculateMappings(HashMap<ONNode, ONNode> mapping, Rule onRule, ObjectNet net) {
		Vector<HashMap<ONNode,ONNode>> onMappings = new Vector<HashMap<ONNode,ONNode>>();
		
		ObjectNet lhs = onRule.getLhs();
		
		GraTra transformation = new DefaultGraTraImpl();
		GraGra grammar = new GraGra(typeSet);
		graph = grammar.getGraph();
		this.net = net;
		transformation.setGraGra(grammar);
		agg.xt_basis.Rule aggRule = grammar.createRule();
		net2graph(net, graph, net2graphNodeMap, net2graphArcMap,false);
		HashMap<ONNode,Node> lhsMap = onRule2aggRule(onRule, aggRule);
			
		Match aggMatch = transformation.createMatch(aggRule);
		onMapping2aggMapping(mapping, aggMatch, lhsMap, net2graphNodeMap);
		//aggMatch.setCompletionStrategy(transformation.getCompletionStrategy(), true);

		while (aggMatch.nextCompletion()) {
			HashMap<ONNode,ONNode> currentMapping = new HashMap<ONNode,ONNode>();
			currentMapping = createONMapping(aggMatch, lhs, net, lhsMap, net2graphNodeMap);
			onMappings.add(currentMapping);
		}

		return onMappings;
	}
	
	/**
	 * Checks whether the specified rule is applicable to the object net. 
	 * 
	 * @param onRule An object net rule that should be applied to an object net.
	 * @param net The object net the rule should be applied to.
	 * @return true, if the rule is applicable to the object net
	 */
	public boolean isApplicable(Rule onRule, ObjectNet net) {
		return isApplicable(null, onRule, net);
	}

	
	/**
	 * Checks whether the specified rule is applicable to the object net. 
	 *
 	 * @param mapping A mapping between ONNodes in the LHS and the object net the rule should be applied to.
	 * @param onRule An object net rule that should be applied to an object net.
	 * @param net The object net the rule should be applied to.
	 * @return true, if the rule is applicable to the object net
	 */
	public boolean isApplicable(HashMap<ONNode, ONNode> mapping, Rule onRule, ObjectNet net) {
		if(!executed) {
			GraTra transformation = new DefaultGraTraImpl();
			GraGra grammar = new GraGra(typeSet);
			graph = grammar.getGraph();
			this.net = net;
			transformation.setGraGra(grammar);
			agg.xt_basis.Rule aggRule = grammar.createRule();
			net2graph(net, graph, net2graphNodeMap, net2graphArcMap,false);
			HashMap<ONNode,Node> lhsMap = onRule2aggRule(onRule, aggRule);
			
			Match aggMatch = transformation.createMatch(aggRule);
			onMapping2aggMapping(mapping, aggMatch, lhsMap, net2graphNodeMap);
			
			boolean result = transformation.apply(aggRule);
			graph.clear();
			
			return result;
		}
		
		return false;
	}
	
	/**
	 * This method will convert an object net instance to an AGG graph.
	 * 
	 * @param net The object net which should be converted.
	 * @param graph A preferably empty graph where the result will be stored.
	 * @param net2graphNodeMap Stores the correspondence between petri net places/transitions and agg nodes (can be null)
	 * @param net2graphArcMap Stores the correspondence between petri net arcs and agg arcs (can be null)
	 * @param isPartofRule true, if given ObjectNet is part of a Rule
	 */
	private void net2graph(ObjectNet net, Graph graph, HashMap<ONNode,Node> net2graphNodeMap, HashMap<ONArc,Arc> net2graphArcMap, Boolean isPartofRule){
		if (net2graphNodeMap != null)
			net2graphNodeMap.clear();
		if (net2graphArcMap != null)
			net2graphArcMap.clear();
		
		try{
		for (Object it: net.getNodes()) {
			ONNode onNode = (ONNode) it;
			Node aggNode = null; 
			if (onNode instanceof ONPlace) {
				aggNode = graph.createNode(typeSet.getTypeForName("Place"));
				ONPlace onPlace = (ONPlace) onNode;
				if (onPlace.getName() != null && onPlace.getName().compareTo("") != 0) {
					aggNode.getAttribute().setExprValueAt("\""+ onPlace.getName()+"\"", "name");
				} else if (!isPartofRule) {
					aggNode.getAttribute().setExprValueAt("\"\"", "name");
				}
				if (onPlace.getTokens() > 0) {
					aggNode.getAttribute().setExprAt(new Integer(onPlace.getTokens()).toString(), "token");
				} else if (!isPartofRule) {
					aggNode.getAttribute().setExprAt(new Integer(onPlace.getTokens()).toString(), "token");
				}
				//if(onPlace.getCapacity() >= 1)
				//	aggNode.getAttribute().setExprAt(new Integer(onPlace.getCapacity()).toString(), "capacity");
			}
			else if (onNode instanceof ONTransition) {
				aggNode = graph.createNode(typeSet.getTypeForName("Transition"));
				if (onNode.getName() != null && onNode.getName().compareTo("") != 0) {
					aggNode.getAttribute().setExprValueAt("\""+onNode.getName()+"\"", "name");
				} else if (!isPartofRule) {
					aggNode.getAttribute().setExprValueAt("\"\"", "name");
				}
			}
				
			
			if (aggNode != null && net2graphNodeMap != null) 
				net2graphNodeMap.put(onNode, aggNode);
		}
		
		for (Object it: net.getArcs()) {
			ONArc onArc = (ONArc) it;
			Arc aggArc = null;
			ONNode onSource = onArc.getSource();
			ONNode onTarget = onArc.getTarget();
			Node aggSource = net2graphNodeMap.get(onSource);
			Node aggTarget = net2graphNodeMap.get(onTarget);
			if (onSource instanceof ONPlace)
				aggArc = graph.createArc(typeSet.getTypeForName("arcPT"), aggSource, aggTarget);
			else if (onSource instanceof ONTransition)
				aggArc = graph.createArc(typeSet.getTypeForName("arcTP"), aggSource, aggTarget);
			
			aggArc.getAttribute().setExprAt(new Integer(onArc.getWeight()).toString(), "weight");
			
			if (aggArc != null && net2graphArcMap != null) {
				net2graphArcMap.put(onArc, aggArc);
			}
		}
		}
		catch (Exception e) {}
	}

	/**
	 * converts a petri net rule to an AGG rule
	 * @param onRule The petri net rule that will be converted.
	 * @param aggRule A preferably empty rule where the result will be stored.
	 * @return HashMap of corresponding nodes between the ONRule LHS and the AGG rule LHS
	 */
	private HashMap<ONNode,Node> onRule2aggRule(Rule onRule, agg.xt_basis.Rule aggRule) {
		HashMap<ONNode,Node> lhsNodeMap = new HashMap<ONNode,Node>();
		HashMap<ONNode,Node> codomainNodeMap = new HashMap<ONNode,Node>();
		Vector<Node> nodes = new Vector<Node>();
		Vector<Arc> arcs = new Vector<Arc>(); 
		
		//convert LHS
		LHS onLHS = onRule.getLhs();
		net2graph(onLHS, aggRule.getSource(), lhsNodeMap, null, true);
		
		//convert RHS and set node mapping
		RHS onRHS = onRule.getRhs();
		net2graph(onRHS, aggRule.getTarget(), codomainNodeMap, null, true);
		for (Object it: onRHS.getMappings()) {
			Mapping mapping = (Mapping) it;
			GraphObject source = lhsNodeMap.get(mapping.getSource());
			GraphObject target = codomainNodeMap.get(mapping.getTarget());
			aggRule.addMapping(source, target);
		}
		//set arc mapping
		Enumeration en = aggRule.getDomain();
		while (en.hasMoreElements()) {
			nodes.add((Node)en.nextElement());
		}
		en = aggRule.getSource().getArcs();
		while (en.hasMoreElements()) {
			arcs.add((Arc)en.nextElement());
		}
		
		aggRule.nextCompletion(nodes.elements(), arcs.elements());
		
		//creates two NACs for each transition to ensure the P/T morphism property 
		createMorphismNACs(aggRule);
		
		//convert NACs and set mapping
		for (Object it: onRule.getNacs()){
			NAC onNAC = (NAC) it;
			OrdinaryMorphism nacMorphism = aggRule.createNAC();
			net2graph(onNAC, nacMorphism.getTarget(), codomainNodeMap, null, true);
			for (Object it2: onNAC.getMappings()) {
				Mapping mapping = (Mapping) it2;
				GraphObject source = lhsNodeMap.get(mapping.getSource());
				GraphObject target = codomainNodeMap.get(mapping.getTarget());
				nacMorphism.addMapping(source, target);
			}
			//set arc mapping
			nodes.clear();
			arcs.clear();
			en = nacMorphism.getDomain();
			while (en.hasMoreElements()) {
				nodes.add((Node)en.nextElement());
			}
			en = nacMorphism.getSource().getArcs();
			while (en.hasMoreElements()) {
				arcs.add((Arc)en.nextElement());
			}
			
			nacMorphism.nextCompletion(nodes.elements(), arcs.elements());
		}
		return lhsNodeMap;
	}
	
	/**
	 * Creates a mapping between a LHS of an AGG rule and an AGG graph based on a mapping defined between ONNodes
	 * 
	 * @param onMapping corresponding ONNodes in a LHS and in an object net
	 * @param aggMapping initially empty AGG match that will be updated according to the onMapping
	 * @param lhsMap relation between the ONRule LHS and the AGG LHS
	 * @param graphMap relation between the object net and the AGG graph
	 */
	private void onMapping2aggMapping(HashMap<ONNode, ONNode> onMapping, Match aggMapping, HashMap<ONNode,Node> lhsMap, HashMap<ONNode,Node> graphMap) {
		if (onMapping != null) {
			Collection<ONNode> onDomain = onMapping.keySet();
			for (ONNode source: onDomain) {
				ONNode target = onMapping.get(source);
				aggMapping.addMapping(lhsMap.get(source),graphMap.get(target));
			}
		}
	}
	
	/**
	 * A petrinet morphism requires that the environment around each transition is preserved, 
	 * meaning that the sum of all incoming arcs must remain constant (same for outoging arcs).
	 * This method adds NACs that ensure that the matching morphism will be a valid petrinet morphism.
	 * 
	 * @param aggRule The rule the NACs should be added to.
	 */
	private void createMorphismNACs(agg.xt_basis.Rule aggRule) {
		Enumeration en = aggRule.getSource().getNodes();
		int i = 0;
		while (en.hasMoreElements()) {
			Node aggNode = (Node) en.nextElement();
			if (aggNode.getType().getStringRepr().equals("Transition")) {
				i++;
				OrdinaryMorphism preNac = aggRule.createNAC();
				preNac.setName("PreTransitionCheck"+i);
				Graph preGraph = preNac.getImage();
				try {
					Node prePlace = preGraph.createNode(typeSet.getTypeForName("Place"));
					Node preTransition = preGraph.createNode(aggNode.getType());
					preGraph.createArc(typeSet.getTypeForName("arcPT"), prePlace, preTransition);
					preNac.addMapping(aggNode, preTransition);
				}catch (Exception e){}
				
				OrdinaryMorphism postNac = aggRule.createNAC();
				postNac.setName("PostTransitionCheck"+i);
				Graph postGraph = postNac.getImage();
				try {
					Node postTransition = postGraph.createNode(typeSet.getTypeForName("Transition"));
					Node postPlace = postGraph.createNode(typeSet.getTypeForName("Place"));
					postGraph.createArc(typeSet.getTypeForName("arcTP"), postTransition, postPlace);
					postNac.addMapping(aggNode, postTransition);
				}catch (Exception e){}
			}
		}
	}
	
	/**
	 * Updates an object net according to an AGG graph. Note that the graph will be completly rebuild if no HashMaps are specified. 
	 * @param graph The graph corresponding with the object net.
	 * @param net2graphNodeMap HashMap with corresponding places/transitions and AGG nodes. 
	 */
	private void updateObjectNet(ObjectNet net, Graph graph, HashMap<ONNode,Node> net2graphNodeMap, HashMap<ONArc,Arc> net2graphArcMap) {
		HashMap<Node,ONNode> graph2netNodeMap = new HashMap<Node,ONNode>();
		HashMap<Arc,ONArc> graph2netArcMap = new HashMap<Arc,ONArc>();
		
		//delete old nodes
		for (Object it: new Vector(net.getNodes())) {
			ONNode onNode = (ONNode) it;
			if(net2graphNodeMap.get(onNode) != null) {
				Node aggNode = net2graphNodeMap.get(onNode);
				if (aggNode.getContext() != null)
					graph2netNodeMap.put(aggNode, onNode);
				else {
					onNode.setObjectNet(null);
					deletedNodes.add(onNode);
				}
			}
			else {
				onNode.setObjectNet(null);
				deletedNodes.add(onNode);
			}
		}
		
		//create new nodes
		Enumeration en = graph.getNodes();
		int i = 0;
		while (en.hasMoreElements()) {
			Node aggNode = (Node) en.nextElement();
			i++;
			if (graph2netNodeMap.get(aggNode) == null) {
				if (aggNode.getType().getStringRepr().equals("Place")) {
					ONPlace newPlace = ronFactory.createONPlace();
					newPlace.setObjectNet(net);
					Integer temp = null;
					if ((temp= ((Integer)aggNode.getAttribute().getValueAt("token"))) != null)
						newPlace.setTokens(temp.intValue());
					else 
						newPlace.setTokens(TOKEN);
					//newPlace.setCapacity(((Integer)aggNode.getAttribute().getValueAt("capacity")).intValue());
					//newPlace.setCapacity(CAPACITY);
					if (aggNode.getAttribute().getValueAt("name") != null)
						newPlace.setName((String)aggNode.getAttribute().getValueAt("name"));
					else
						newPlace.setName("newPlace"+i);
					newPlace.setX(i*50);
					newPlace.setY(100);
					
					graph2netNodeMap.put(aggNode, newPlace);
					createdNodes.add(newPlace);
				} else {
					ONTransition newTransition = ronFactory.createONTransition();
					newTransition.setObjectNet(net);
					newTransition.setName((String)aggNode.getAttribute().getValueAt("name"));
					newTransition.setX(i*50);
					newTransition.setY(100);
					
					graph2netNodeMap.put(aggNode, newTransition);
					createdNodes.add(newTransition);
				}
			}
		}
		
		//delete old arcs
		for (Object it: new Vector(net.getArcs())){
			ONArc onArc = (ONArc) it;
			if(net2graphNodeMap.get(onArc) != null) {
				Arc aggArc = net2graphArcMap.get(onArc);
				graph2netArcMap.put(aggArc, onArc);
			}
			else {
				deletedArcs.add(onArc.getSource());
				deletedArcs.add(onArc);
				deletedArcs.add(onArc.getTarget());
				
				onArc.setSource(null);
				onArc.setTarget(null);
				onArc.setObjectNet(null);
			}
		}

		//create new arcs
		Enumeration en2 = graph.getArcs();
		while (en2.hasMoreElements()) {
			Arc aggArc = (Arc) en2.nextElement();
			if (graph2netArcMap.get(aggArc) == null) {
				ONArc newArc = ronFactory.createONArc();
				if (aggArc.getAttribute().getValueAt("weight") != null)
					newArc.setWeight(((Integer)aggArc.getAttribute().getValueAt("weight")).intValue());
				else 
					newArc.setWeight(WEIGHT);
				
				ONNode source = graph2netNodeMap.get(aggArc.getSource());
				ONNode target = graph2netNodeMap.get(aggArc.getTarget());
				
				createdArcs.add(source);
				createdArcs.add(newArc);
				createdArcs.add(target);
				
				newArc.setObjectNet(net);
				newArc.setSource(source);
				newArc.setTarget(target);
			}
		}
	}
	
	private HashMap<ONNode,ONNode> createONMapping(Match aggMatch, ObjectNet source, ObjectNet target, HashMap<ONNode,Node> sourceMap, HashMap<ONNode,Node> targetMap) {
		HashMap<ONNode,ONNode> onMapping = new HashMap<ONNode,ONNode>();
		for (Object it: source.getNodes()) {
			ONNode sourceONNode = (ONNode) it;
			Node sourceAGGNode = sourceMap.get(sourceONNode);
			Node targetAGGNode = (Node) aggMatch.getImage(sourceAGGNode);
			ONNode targetONNode = getONNodeByAggNode(targetAGGNode, targetMap);
			onMapping.put(sourceONNode, targetONNode);
		}
		return onMapping;
	}
	

	private ONNode getONNodeByAggNode(Node aggNode, HashMap<ONNode,Node> map) {
		for (Object it: map.keySet()) {
			ONNode onNode = (ONNode) it;
			if (map.get(onNode) == aggNode)
				return onNode;
		}
		return null;
	}
	
	/**
	 * Undoes the rule application.
	 *
	 */
	public void undo() {
		if(executed && !undone) {
			// create deleted nodes
			for (int i=0; i < deletedNodes.size(); i++) {
				ONNode node = (ONNode) deletedNodes.get(i);
				node.setObjectNet(net);
			}
			
			// create deleted arcs
			for (int i=0; i < deletedArcs.size(); i=i+3) {
				ONNode source = (ONNode) deletedArcs.get(i);
				ONArc arc = (ONArc) deletedArcs.get(i+1);
				ONNode target = (ONNode) deletedArcs.get(i+2);
				arc.setObjectNet(net);
				arc.setSource(source);
				arc.setTarget(target);
			}
			
			// delete created arcs
			for (int i=0; i < createdArcs.size(); i=i+3) {
				ONArc arc = (ONArc) createdArcs.get(i+1);
				arc.setSource(null);
				arc.setTarget(null);
				arc.setObjectNet(null);
			}
			
			// delete created nodes
			for (int i=0; i < createdNodes.size(); i++) {
				ONNode node = (ONNode) createdNodes.get(i);
				node.setObjectNet(null);
			}

			undone = true;
		}	
	}
	
	/**
	 * Redoes the rule application.
	 *
	 */
	public void redo() {
		if (undone) {
			// recreate nodes
			for (int i=0; i < createdNodes.size(); i++) {
				ONNode node = (ONNode) createdNodes.get(i);
				node.setObjectNet(net);
			}
			
			// recreate arcs
			for (int i=0; i < createdArcs.size(); i=i+3) {
				ONNode source = (ONNode) createdArcs.get(i);
				ONArc arc = (ONArc) createdArcs.get(i+1);
				ONNode target = (ONNode) createdArcs.get(i+2);
				arc.setObjectNet(net);
				arc.setSource(source);
				arc.setTarget(target);
			}
			
			// delete arcs
			for (int i=0; i < deletedArcs.size(); i=i+3) {
				ONArc arc = (ONArc) deletedArcs.get(i+1);
				arc.setSource(null);
				arc.setTarget(null);
				arc.setObjectNet(null);
			}
			
			// delete nodes
			for (int i=0; i < deletedNodes.size(); i++) {
				ONNode node = (ONNode) deletedNodes.get(i);
				node.setObjectNet(null);
			}
			
			undone = false;
		}
	}
}