package roneditor.dialogs.match;

import java.util.ArrayList;

import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Tool;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.MenuManager;

import roneditor.model.ModelCreationFactory;
import roneditor.objectnetview.MappingCreationTool;
import roneditor.objectnetview.commands.DeleteMappingCommand;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import roneditor.objectnetview.edit.utils.ObjectNetViewEditPartFactory;
import roneditor.ruleview.RulePageContextMenuProvider;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;

/**
 * Subclasses of AbstractMatchDialog are meant to adapt AbstractMatchDialog to a
 * concrete RONEditor by implementing the following methods.
 * 
 * @author Tony Modica
 */
public class MatchDialog extends AbstractMatchDialog {

	/**
	 * Main constructor for this dialog.
	 * 
	 * @param rules
	 *            The rules that can be switched through for match-setting. Must
	 *            contain at least 1 rule.
	 * @param objectNets
	 *            The target object nets that can be switched through for
	 *            match-setting. Must contain at least 1 object net.
	 */
	public MatchDialog(final ArrayList<Rule> rules,
			final ArrayList<ObjectNet> objectNets) {
		super(rules, objectNets);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createContextMenuProvider(org.eclipse.gef.GraphicalViewer,
	 *      org.eclipse.gef.ui.actions.ActionRegistry)
	 */
	@Override
	protected MenuManager createContextMenuProvider(
			final GraphicalViewer viewer, final ActionRegistry actionRegistry) {
		return new RulePageContextMenuProvider(viewer, actionRegistry);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetViewEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#createMappingTool()
	 */
	@Override
	protected Tool createMappingTool() {
		return new MappingCreationTool(
				new ModelCreationFactory(Mapping.class));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#buildCreateMappingCommand(ronmodel.Codomain,
	 *      ronmodel.ONNode, ronmodel.ONNode)
	 */
	@Override
	protected Command buildCreateMappingCommand(final Codomain codomain,
			final ONNode source, final ONNode target) {
			if (source instanceof ONPlace && target instanceof ONPlace) {
				return buildCreateONPlaceMappingCommand(codomain,(ONPlace)source,(ONPlace)target);
			} else if (source instanceof ONTransition && target instanceof ONTransition) {
				return buildCreateONTransitionMappingCommand(codomain,(ONTransition)source,(ONTransition)target);
			}
			return null;
	}
	
	private Command buildCreateONPlaceMappingCommand(final Codomain codomain,
			final ONPlace source, final ONPlace target) {
		 final MappingONPlaceCommand createCommand = new
		 MappingONPlaceCommand();
		 createCommand.setMapping(RonmodelFactory.eINSTANCE.createMapping());
		 createCommand.setSource(source);
		 createCommand.setTarget(target);
		 return createCommand;
		
	}
	
	private Command buildCreateONTransitionMappingCommand(final Codomain codomain,
			final ONTransition source, final ONTransition target) {
		 final MappingONTransitionCommand createCommand = new
		 MappingONTransitionCommand();
		 createCommand.setMapping(RonmodelFactory.eINSTANCE.createMapping());
		 createCommand.setSource(source);
		 createCommand.setTarget(target);
		 return createCommand;
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.dialogs.match.AbstractMatchDialog#buildDeleteMappingCommand(ronmodel.Mapping)
	 */
	@Override
	protected Command buildDeleteMappingCommand(final Mapping mapping) {
		 final DeleteMappingCommand deleteCommand = new
		 DeleteMappingCommand();
		 deleteCommand.setMapping(mapping);
		 return deleteCommand;
	}
}
