package roneditor.edit.parts;

import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import org.eclipse.gef.tools.DragTreeItemsTracker;
import org.eclipse.gef.tools.SelectEditPartTracker;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.IRonEditorConstants;
import roneditor.AbstractTreeEditor.EditorTreeViewer;
import roneditor.actions.CreateHLPlaceAction;
import roneditor.model.RONModelManager;
import roneditor.model.RONPropertySource;
import roneditor.ronview.commands.CreateHLPlaceCommand;
import ronmodel.RON;
import ronmodel.RonmodelPackage;
import _mapping_consistency.MappingConsistencyCheck;

/**
 * TreeEditPart for the top-level container item in the tree editor. This
 * provides an visual item in the tree editor and allows the execution of
 * actions to create HLPlaces and HLTransitions via context menu on this item.
 * 
 * @author Tony Modica
 */
public class RONTreeEditPart extends AbstractTreeEditPart implements Adapter {

	/**
	 * Target field for {@link Adapter} implementation.
	 */
	private Notifier target;

	private IPropertySource propertySource;

	public RONTreeEditPart(final Object model) {
		super(model);
	}

	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		final Object type = req.getType();
		if (type.equals(RequestConstants.REQ_OPEN)) {
			openRONView();
		}
	}
	
	/**
	 * Opens a View for editing the object net that is this editpart's model.
	 */
	private void openRONView() {
		((EditorTreeViewer)getViewer()).getEditor().showView(IRonEditorConstants.ronViewID,getCastedModel());
	}
		
	/**
	 * Returns a List of all HLNodes in the RON which is the model of this
	 * EditPart.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List<Object> getModelChildren() {
		return getCastedModel().getNodes();
	}

	/**
	 * Convenience method that casts the model to the appropriate type.
	 * 
	 * @return The model of this EditPart appropriately casted.
	 */
	public RON getCastedModel() {
		return (RON) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return null;
	}

	/**
	 * Besides the requests handled by the installed EditPolicies (none for this
	 * EditPart) this EditPart understands following request.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	@Override
	public boolean understandsRequest(final Request req) {
		final Object type = req.getType();
		return type.equals(CreateHLPlaceAction.ID)
				|| super.understandsRequest(req);
	}

	/**
	 * Returns a Command if the appropriate request is passed. Checks the
	 * Commands handled by installed EditPolicies (none for this EditPart) as
	 * well.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	@Override
	public Command getCommand(final Request request) {
		if ((CreateHLPlaceAction.ID).equals(request.getType())) {
			CreateHLPlaceCommand create=new CreateHLPlaceCommand();
			create.setRON(getCastedModel());
			create.setHLPlace(RONModelManager.getFactory().createHLPlace());
			return create;
			
		}
		return super.getCommand(request);
	}

	/**
	 * Hooks in this EditPart as a listener for changes of its model.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	@Override
	public void activate() {
		super.activate();
		getCastedModel().eAdapters().add(this);
	}

	/**
	 * Hooks off this EditPart as a listener for changes of its model.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	@Override
	public void deactivate() {
		super.deactivate();
		getCastedModel().eAdapters().remove(this);
	}

	/**
	 * Returns a DragTracker to handle direct editing and/or opening. As this
	 * EditPart is not represented as an {@link TreeItem} a
	 * {@link SelectEditPartTracker} instead of a {@link DragTreeItemsTracker}
	 * is returned, so that this EditPart can be selected with a click on the
	 * background.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getDragTracker(org.eclipse.gef.Request)
	 */
	@Override
	public DragTracker getDragTracker(final Request req) {
		return new SelectEditPartTracker(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(final Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @return A RONPropertySource for the Properties View.
	 */
	private IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new RONPropertySource(getCastedModel());
		}
		return propertySource;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(final Object type) {
		return type.equals(getModel().getClass());
	}

	/**
	 * If the model fires a property change notification to its listeners, the
	 * visual elements which are managed by this EditPart will be refrehed
	 * depending on the featureID the notifiation contains.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		// HINT Hier wird auf Vernderung bestimmter Features aus dem EMF-Modell
		// reagiert
		RON ron=(RON)getModel();
		IResource res=(IResource) ((EditorTreeViewer)getViewer()).getEditor().getAdapter(IResource.class);
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.RON__NODES:
			refreshChildren();
			MappingConsistencyCheck.deleteAllMarkersOfNonExistingRules(ron, res);
			MappingConsistencyCheck.checkMappingConsistencyOfRulesWithoutAnyMarker(ron, res);
			break;
		case RonmodelPackage.RON__NAME:
			refreshVisuals();
			break;
		default:
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(final Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}
}
