/**
 * 
 */
package roneditor.ronview.edit.policies;


import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;

import roneditor.ronview.commands.AbstractHLConnectionCommand;
import roneditor.ronview.commands.ReconnectHLArcCommand;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;

/**
 * @author Alex
 *
 */
public abstract class HLNodeEditPolicy extends GraphicalNodeEditPolicy {

	
	/**
	 * Checks if there already exist a connection between source and target.
	 * @param onArc the onArc to compare with (ReconnectCommand) or null (otherwise)
	 * @param source the source
	 * @param target the target
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	private static boolean thereIsAlreadyHLArc(HLArc onArc, HLNode source, HLNode target) {
		if (source==null || target==null) return false;
		for (Object o:source.getOutgoing()){
			HLArc arc=((HLArc)o); 
			if (arc.getTarget()==target && arc!=onArc) return true;			
		}
		return false;		
	}
	
	/**
	 * Checks if there already exist a connection between source and target.
	 * @param command AbstractConnectionCommand
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	protected static boolean thereIsAlreadyHLArc(AbstractHLConnectionCommand command) {
		HLNode source=command.getSource();
		HLNode target=command.getTarget();
		return thereIsAlreadyHLArc(null,source, target);
	}

	/**
	 * Checks if there already exist a connection between source and target.
	 * @param command ReconnectHLArcCommand
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	protected static boolean thereIsAlreadyHLArc(ReconnectHLArcCommand command) {
		HLArc onArc=command.getHLArc();
		HLNode source=command.getNewSource();
		HLNode target=command.getNewTarget();
		if (source==null){
			source=command.getHLArc().getSource();
		} 
		if (target==null){
			target=command.getHLArc().getTarget();
		}
		return thereIsAlreadyHLArc(onArc, source, target);
	}
	
	/**
	 * Checks if the command changes something (important for Undo / Redo)
	 * @param command ReconnectHLArcCommand
	 * @return true if the command changes something, false otherwise
	 */
	protected static boolean nothingChanged(ReconnectHLArcCommand command){
		HLNode oldSource=command.getOldSource();
		HLNode oldTarget=command.getOldTarget();
		HLNode newSource=command.getNewSource();
		HLNode newTarget=command.getNewTarget();
		int newAnchorNrS=command.getNewAnchorNrS();
		int newAnchorNrT=command.getNewAnchorNrT();
		if (newSource==null){
			newSource=command.getHLArc().getSource();
			newAnchorNrS=command.getHLArc().getAnchorNrS();
		} 
		if (newTarget==null){
			newTarget=command.getHLArc().getTarget();
			newAnchorNrT=command.getHLArc().getAnchorNrT();
		}
		if (newSource!=oldSource) return false;
		if (newTarget!=oldTarget) return false;
		if (newAnchorNrS!=command.getOldAnchorNrS()) return false;
		if (newAnchorNrT!=command.getOldAnchorNrT()) return false;
		return true;
	}
	
	/**
	 * Counts pre and post places of a HLTransition
	 * @param trans the (HL-)Transition
	 * @return int-array[0..3]: int-array[0]: # pre NET-places, int-array[1]: # pre RULE-places, int-array[2]: # post NET-places, int-array[3]: # post RULE-places
	 */
	private static int[] countPreAndPostPlaces(HLTransition trans){
		int[] i=new int[4];
		//count pre-places
		for (Object o: trans.getIncoming()){
			//for all incoming arcs i.e. for all pre nodes
			HLPlace place=(HLPlace)((HLArc)o).getSource();
			if (place.getType().getValue()==HLPlaceType.NET)
				i[0]++;
			else
				i[1]++;
		}
		//count post-places
		for (Object o: trans.getOutgoing()){
			//for all outgoing arcs i.e. for all post nodes
			HLPlace place=(HLPlace)((HLArc)o).getTarget();
			if (place.getType().getValue()==HLPlaceType.NET)
				i[2]++;
			else
				i[3]++;
		}
		return i;
	}
	
	/**
	 * @author alex
	 * 
	 * @param from source of arc
	 * @param to target of arc
	 * @param isReconnect true if the request is a reconnect-request, false otherwise
	 * @return true if arc can be created
	 */
	protected static boolean isAllowedArc(HLNode from, HLNode to, boolean isReconnect){
		//count pre and post places
		int[] c;
		try{			
			int preNetPlaces=0;
			int preRulePlaces=0;
			int postNetPlaces=0;
			int postRulePlaces=0;
			boolean reqPreNet=false;
			boolean reqPreRule=false;
			boolean reqPostNet=false;
			boolean reqPostRule=false;						
			HLTransition trans;
			if (from instanceof HLTransition && to instanceof HLPlace){
				c=countPreAndPostPlaces(trans=(HLTransition)from);
				if (((HLPlace)to).getType().getValue()==HLPlaceType.NET)
					reqPostNet=true;
				else
					reqPostRule=true;
			} else if (to instanceof HLTransition && from instanceof HLPlace){
				c=countPreAndPostPlaces(trans=(HLTransition)to);
				if (((HLPlace)from).getType().getValue()==HLPlaceType.NET)
					reqPreNet=true;
				else
					reqPreRule=true;
			} else return false;
			preNetPlaces+=c[0];
			preRulePlaces+=c[1];
			postNetPlaces+=c[2];
			postRulePlaces+=c[3];			
			if (trans.getKind()==HLTransitionKind.APPLYRULE_LITERAL){
				if (preRulePlaces>=1 && (reqPreRule && !isReconnect)) return false;	
			}			
			if (trans.getKind()==HLTransitionKind.FIRE_LITERAL){
				if (preNetPlaces>=1 && (reqPreNet && !isReconnect)) return false;
				if (reqPreRule) return false;
				if (reqPostRule) return false;
			}			
			if (trans.getKind()==HLTransitionKind.SPLIT_LITERAL){
				if (preNetPlaces>=1 && (reqPreNet && !isReconnect)) return false;
				if (postNetPlaces>=1 && (reqPostNet && !isReconnect)) return false;
				if (reqPreRule) return false;
				if (reqPostRule) return false;
			}			
			if (trans.getKind()==HLTransitionKind.STANDARD_LITERAL){
				if (preRulePlaces>=1 && (reqPreRule && !isReconnect)) return false;
				if (preRulePlaces==0 && reqPostRule) return false;				
			}			
			
			return true;
		} catch(NullPointerException npe){
			return false;
		}
	}

}
