package roneditor.ruleview;

import org.eclipse.core.resources.IResource;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;

import roneditor.AbstractMultiViewerPage;
import roneditor.AbstractPageBookView;
import roneditor.actions.AddTokenAction;
import roneditor.actions.FireTransitionAction;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.model.ModelCreationFactory;
import roneditor.objectnetview.MappingCreationToolEntry;
import roneditor.objectnetview.ObjectNetPaletteRoot;
import roneditor.objectnetview.actions.AutoLayoutAction;
import roneditor.objectnetview.actions.EditAction;
import roneditor.objectnetview.actions.MoveNodeAction;
import roneditor.objectnetview.edit.utils.ObjectNetViewEditPartFactory;
import roneditor.ronview.actions.CompleteMappingConsistencyCheckAction;
import roneditor.ronview.actions.DeleteHLBendPointAction;
import roneditor.ruleview.actions.CopyLhsToNacAction;
import roneditor.ruleview.actions.CopyLhsToRhsAction;
import roneditor.ruleview.actions.CopyRhsToNacAction;
import roneditor.ruleview.actions.CreateNacAction;
import roneditor.ruleview.actions.DeleteNacAction;
import roneditor.ruleview.actions.DeleteRuleBendPointAction;
import roneditor.ruleview.actions.SwitchNACAction;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import _mapping_consistency.MappingConsistencyCheck;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link Rule}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetViewEditPartFactory} and
 * {@link RulePageContextMenuProvider}, configures the keyhandler and creates
 * actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class RulePage extends AbstractMultiViewerPage {

		
	
	public RulePage(final Rule rule, final IEditorPart editor) {
		super(rule, editor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit der ContextMenuProvider sie in das Kontextmen einfgen kann.

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);

		// A sample action to switch through the NACs
		final Action action = new SwitchNACAction(this);
		getActionRegistry().registerAction(action);
		
		final Action copyAction =  new CopyLhsToRhsAction(this, this.getEditor());
		getActionRegistry().registerAction(copyAction);
		
		final SelectionAction copyLhsAction =  new CopyLhsToNacAction(this, this.getEditor());
		getActionRegistry().registerAction(copyLhsAction);

		final Action copyRhsToNacAction =  new CopyRhsToNacAction(this, this.getEditor());
		getActionRegistry().registerAction(copyRhsToNacAction);
		
		// Create and register the AddToken Action
		final Action addTokenAction =  new AddTokenAction(getEditor());
		getActionRegistry().registerAction(addTokenAction);
		registerHandleForEditorAction(AddTokenAction.ID);
		
		// Create and register the Edit Action
		final Action editAction =  new EditAction(getEditor());
		getActionRegistry().registerAction(editAction);
		registerHandleForEditorAction(EditAction.ID);
		
		
		getActionRegistry().registerAction(new DeleteRuleBendPointAction(getEditor()));
		registerHandleForEditorAction(DeleteRuleBendPointAction.ID);
		
		// Delete Action
		getActionRegistry().registerAction(new DeleteAction(getEditor()));
		registerHandleForEditorAction(DeleteAction.ID);
				
		// Create NAC
		final Action createNacAction =  new CreateNacAction(this, this.getEditor());
		getActionRegistry().registerAction(createNacAction);
		
		// Delete NAC
		final Action deleteNacAction =  new DeleteNacAction(this, this.getEditor());
		getActionRegistry().registerAction(deleteNacAction);
		
		getActionRegistry().registerAction(new CompleteMappingConsistencyCheckAction(getEditor()));
		registerHandleForEditorAction(CompleteMappingConsistencyCheckAction.ID);
		
//		 automatisches Layout
		getActionRegistry().registerAction(new AutoLayoutAction(getEditor()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry()
				.getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry()
				.getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		// Just show the nets like in ObjectNetView, nothing special here
		return new ObjectNetViewEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new RulePageContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// ObjectNet Buttons ( select, marquee, onplace, ontransition, onarc)
		PaletteRoot paletteRoot = new ObjectNetPaletteRoot();
		// Mapping Button
		PaletteGroup mappingGroup = new PaletteGroup("Mapping");
		paletteRoot.add(mappingGroup);
		
		CreationToolEntry entry = new MappingCreationToolEntry("Mapping",
				"Creates a Mapping", new ModelCreationFactory(Mapping.class),
				null, null);

		mappingGroup.add(entry);
		
		
		return paletteRoot;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}
	
//	@Override
//	public void createControl(final Composite parent) {
//		super.createControl(parent);
//		// nach dem alle Visual Parts erstellt wurden, Mappingnummern neu berechnen und setzen
//		// bewirkt das beim erstmaligem Laden der Page die Mappingnummern gesetzt sind
//		roneditor.objectnetview.MappingCreationTool.calculateMapping(this);
//	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		Rule rule=(Rule)getModel();		
		IResource res=(IResource) getEditor().getAdapter(IResource.class);
		switch (featureId) {
			case RonmodelPackage.RULE__NACS:
				MappingConsistencyCheck.deleteAllMarkersOfRule(rule, res);
				MappingConsistencyCheck.checkMappingConsistency(rule.getLhs(), res);
				break;
			default:
				break;
		}
	}
	
	/**
	 * A {@link RulePage} shows three nets: {@link NAC}, {@link LHS} and
	 * {@link RHS} of the {@link Rule}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 3;
	}

	/**
	 * A {@link RulePage} shows the first {@link NAC} of the {@link Rule} and
	 * the {@link LHS} and {@link RHS}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		final Rule rule = (Rule) getModel();
		EObject nac= null;
		if ( rule.getNacs().size() > 0) {
			nac = (EObject) rule.getNacs().get(0);
		} 
		return new EObject[] { nac, rule.getLhs(),
				rule.getRhs() };
	}

	/**
	 * @return The model of the page casted to a {@link Rule}.
	 */
	public Rule getRule() {
		return (Rule) getModel();
	}
}
